/* main.c */
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include "utils.h"
#include <math.h>

static char VERSION[] = "0.2";

static void usage(char *who);
static void print_ps_header(FILE *fp);
static void print_ps_header2(FILE *fp, char*A, char *B, char *C, char *D);

extern double * tree_support( char **A ,int noGaps,int Frame);
extern void print_triangle_data(FILE *fp, int N, double **P);

int main(int argc, char *argv[] ){
  int i,j,k,l,K,QQ,ii;
  int noGaps, Frame, Verbose;    /* options */
  char T1[10000], T2[10000];
  char *TIT[5];
  char *line;
  int *n;
  double m1, m2, m3, mm,  v1, v2, v3, vv;
  char ***A;
  char ***Aname;
  char **AA;
  double *s;
  FILE *fp;
  int N;
  double **P;

  noGaps  = 0;
  Frame   = 0;
  Verbose = 0;

  for (i=1; i<argc; i++) {
    if (argv[i][0]=='-') {
      switch ( argv[i][1] )
        {
	  int r;
        case 'F' : r=sscanf(argv[++i], "%d", &Frame);
          if (!r) usage(argv[0]);
          break;
	case 'g' :  noGaps=1;
	  break;
	case 'v' :  Verbose=1;
	  break;
	case 'V' :
	  printf("%s: quartm, version %s\n",argv[0],VERSION);
	  exit(1);
	  break; 
        case 'h' :
        default:
          usage(argv[0]);
        }
    }
  }
  if((Frame<0)||(Frame>3)) {
    Frame = 0;
    fprintf(stderr,"Warning: Ignoring Frame flag\n");
  }
  
  if(Verbose) {
    printf("# Parameters\n");
    if(noGaps)
      printf("#   ignoring quadruples containing gap character '-'\n");
    else printf("#   treating gap symbol '-' as character\n");
    if(Frame) printf("#   using reading frame %d only\n",Frame);
    else printf("#   using all positions\n");
  }
  
  n = (int *) space(sizeof(int)*5);
  A = (char ***) space(sizeof(char **)*5);
  Aname = (char ***) space(sizeof(char **)*5);
  for(i=0;i<5;i++) A[i] = (char **) space(sizeof(char*)*1999);
  for(i=0;i<5;i++) Aname[i] = (char **) space(sizeof(char*)*1999);
  i=0;j=0;
  TIT[0] = "Query";
  line = get_line(stdin);
  while(line) {
    T1[0]='\0';
    T2[0]='\0';
    if(line[0]=='&') {
      n[i]=j;
      j=0;
      strcat(T2,"?");
      sscanf(line,"%s %s",T1,T2);
      i++;
      TIT[i] = strdup(T2);
      if(i==5) break;
    }
    else {
      sscanf(line,"%s %s",T1,T2);
      Aname[i][j] = strdup(T1);
      A[i][j]=strdup(T2);
      j++;
    }
    free(line);
    line = get_line(stdin);
  }

  /* What have I read? */

  for(i=0;i<5;i++) {
    if(n[i]>0) { 
      printf("%1d : %3d sequences in '%s'\n",i,n[i],TIT[i]);
      for(j=0;j<n[i];j++) {
	printf("  %2d %s\n",j,Aname[i][j]);
      }
    }
  }
  printf("\n");
  N = n[1]*n[2]*n[3]*(1+n[4]);
  P = (double **) space(sizeof(double*)*(N+1));
  for(ii=0;ii<N;ii++) P[ii] = (double*) space(sizeof(double)*2);
      
  
  for(K=0;K<n[0];K++) { 
    printf(">>> Query = '%s'\n",Aname[0][K]);

    TIT[0] = Aname[0][K];
    
    if(n[4]==0) { 
      
      T1[0]='\0';
      T2[0]='\0';
      strcat(T1,Aname[0][K]);
      if(Frame) {
	T2[0]='\0';
	fprintf(stderr,"%d\n",Frame);
	sprintf(T2,"-f%d",Frame);
	strcat(T1,T2);
      }
      strcat(T1,"-tri.ps");
      printf(" -> %s\n",T1);
      fp=fopen(T1,"w");
      AA = (char **) space(sizeof(char*)*5);
      print_ps_header(fp);
      print_ps_header2(fp,TIT[0],TIT[1],TIT[2],TIT[3]);
      
      m1=0.0; m2=0.0; m3=0.0; v1=0.0; v2=0.0; v3=0.0;
      ii=0;
      for(j=0;j<n[1];j++) {
	for(k=0;k<n[2];k++) {
	  for(l=0;l<n[3];l++) {
	    AA[0] = A[0][K];
	    AA[1] = A[1][j];
	    AA[2] = A[2][k];
	    AA[3] = A[3][l];
	    s = tree_support(AA,noGaps,Frame);
	    P[ii][0] = s[0]; P[ii][1]=s[1]; ii++;
	    m1 += s[0];      m2 += s[1];      m3 += s[2];
	    v1 += s[0]*s[0]; v2 += s[1]*s[1]; v3 += s[2]*s[2]; 
	    free(s);
	  }
	}
      }
      free(AA);
      print_triangle_data(fp,ii,P);
      /* statistics to stdout */
      mm = (double) ii;
      m1=m1/mm; m2=m2/mm; m3=m3/mm;
      v1=v1/mm; v2=v2/mm; v3=v3/mm; 
      vv = (v1+v2+v3)/3.0;
      printf("T1: %6.4f +/- %6.4f   [(%s,%s)(%s,%s)]\n",
	     m1, sqrt(v1-m1*m1) ,TIT[0],TIT[1],TIT[2],TIT[3]);
      printf("T2: %6.4f +/- %6.4f   [(%s,%s)(%s,%s)]\n",
	     m2, sqrt(v2-m2*m2) ,TIT[0],TIT[2],TIT[1],TIT[3]);
      printf("T3: %6.4f +/- %6.4f   [(%s,%s)(%s,%s)]\n",
	     m3, sqrt(v3-m3*m3) ,TIT[0],TIT[3],TIT[1],TIT[2]);
            
      fclose(fp);
    }
    else {
      char Aa[1000], Bb[1000], Cc[1000], Dd[1000];
      for(QQ=1;QQ<=3;QQ++) {
	T1[0]='\0';
	strcat(T1,Aname[0][K]);
	if(Frame) {
	  T2[0]='\0';
	  sprintf(T2,"-f%d",Frame);
	  strcat(T1,T2);
	}
	sprintf(T2,"-tri-%d.ps",QQ);	
	strcat(T1,T2);
	printf(" -> %s\n",T1);
	fp=fopen(T1,"w");
	AA = (char **) space(sizeof(char*)*5);
	print_ps_header(fp);
	
	m1=0.0; m2=0.0; m3=0.0;
	v1=0.0; v2=0.0; v3=0.0;
	ii=0; mm=0.0; 
	for(j=0;j<n[1];j++) {

	  for(k=0;k<n[2];k++) {
	    for(l=0;l<n[3];l++) {
	      int i1;
	      AA[0] = A[0][K];
	      AA[1] = A[1][j];
	      AA[2] = A[2][k];
	      AA[3] = A[3][l];
	      s = tree_support(AA,noGaps,Frame);
	      P[ii][0] = s[0]; P[ii][1]=s[1]; ii++;
	      for(i1=0;i1<3;i1++) {
		fprintf(fp, "%8.6f ",s[i1]);
	      }
	      mm += 1.0;
	      m1 += s[0];      m2 += s[1];      m3 += s[2];
	      v1 += s[0]*s[0]; v2 += s[1]*s[1]; v3 += s[2]*s[2]; 
	      fprintf(fp, "sink\n");
	      free(s);
	    }
	  }
	}
	if(QQ==1) {
	  sprintf(Aa,"%s",TIT[0]);
	  sprintf(Bb,"%s+%s",TIT[1],TIT[4]);
	  sprintf(Cc,"%s",TIT[2]);
	  sprintf(Dd,"%s",TIT[3]);
	  print_ps_header2(fp,Aa,Bb,Cc,Dd);
	  for(j=0;j<n[4];j++) {
	    for(k=0;k<n[2];k++) {
	      for(l=0;l<n[3];l++) {
		int i1;
		AA[0] = A[0][K];
		AA[1] = A[4][j];
		AA[2] = A[2][k];
		AA[3] = A[3][l];
		s = tree_support(AA,noGaps,Frame);
		P[ii][0] = s[0]; P[ii][1]=s[1]; ii++;
		for(i1=0;i1<3;i1++) {
		  fprintf(fp, "%8.6f ",s[i1]);
		}
		mm += 1.0;
		m1 += s[0];      m2 += s[1];      m3 += s[2];
		v1 += s[0]*s[0]; v2 += s[1]*s[1]; v3 += s[2]*s[2]; 
		fprintf(fp, "sink\n");
		free(s);
	      }
	    }
	  }
	}
	if(QQ==2) {
	  sprintf(Aa,"%s",TIT[0]);
	  sprintf(Bb,"%s",TIT[1]);
	  sprintf(Cc,"%s+%s",TIT[2],TIT[4]);
	  sprintf(Dd,"%s",TIT[3]);
	  print_ps_header2(fp,Aa,Bb,Cc,Dd);
	  for(j=0;j<n[1];j++) {
	    for(k=0;k<n[4];k++) {
	      for(l=0;l<n[3];l++) {
		int i1;
		AA[0] = A[0][K];
		AA[1] = A[1][j];
		AA[2] = A[4][k];
		AA[3] = A[3][l];
		s = tree_support(AA,noGaps,Frame);
		P[ii][0] = s[0]; P[ii][1]=s[1]; ii++;
		for(i1=0;i1<3;i1++) {
		  fprintf(fp, "%8.6f ",s[i1]);
		}
		mm += 1.0;
		m1 += s[0];      m2 += s[1];      m3 += s[2];
		v1 += s[0]*s[0]; v2 += s[1]*s[1]; v3 += s[2]*s[2]; 
		fprintf(fp, "sink\n");
		free(s);
	      }
	    }
	  }
	}
	if(QQ==3) {
	  sprintf(Aa,"%s",TIT[0]);
	  sprintf(Bb,"%s",TIT[1]);
	  sprintf(Cc,"%s",TIT[2]);
	  sprintf(Dd,"%s+%s",TIT[3],TIT[4]);
	  print_ps_header2(fp,Aa,Bb,Cc,Dd);
	  for(j=0;j<n[1];j++) {
	    for(k=0;k<n[2];k++) {
	      for(l=0;l<n[4];l++) {
		int i1;
		AA[0] = A[0][K];
		AA[1] = A[1][j];
		AA[2] = A[2][k];
		AA[3] = A[4][l];
		s = tree_support(AA,noGaps,Frame);
		P[ii][0] = s[0]; P[ii][1]=s[1]; ii++;
		for(i1=0;i1<3;i1++) {
		  fprintf(fp, "%8.6f ",s[i1]);
		}
		mm += 1.0;
		m1 += s[0];      m2 += s[1];      m3 += s[2];
		v1 += s[0]*s[0]; v2 += s[1]*s[1]; v3 += s[2]*s[2]; 
		fprintf(fp, "sink\n");
		free(s);
	      }
	    }
	  }
	}
	free(AA);
	print_triangle_data(fp,ii,P);
	m1=m1/mm; m2=m2/mm; m3=m3/mm;
	v1=v1/mm; v2=v2/mm; v3=v3/mm; 
	vv = (v1+v2+v3)/3.0;
	printf("QQ=%d\n",QQ);
	printf("T1: %6.4f +/- %6.4f  %s", m1, sqrt(v1-m1*m1),TIT[1] );
	if(QQ==1) printf(" + %s\n",TIT[4]);else printf("\n");
	printf("T2: %6.4f +/- %6.4f  %s", m2, sqrt(v2-m2*m2),TIT[2] );
	if(QQ==2) printf(" + %s\n",TIT[4]);else printf("\n");
	printf("T3: %6.4f +/- %6.4f  %s", m3, sqrt(v3-m3*m3),TIT[3] );
	if(QQ==3) printf(" + %s\n",TIT[4]);else printf("\n");
		
	fprintf(fp,"0 1 0 setrgbcolor\nnewpath\n");
	fprintf(fp,"%6.4f %6.4f %6.4f tricoord 0.015 0 360 arc\n",m1,m2,m3);
	fprintf(fp,"fill stroke\nnewpath\n");
	fprintf(fp,"%6.4f %6.4f %6.4f tricoord %6.4f 0 360 arc\n",m1,m2,m3,vv);
	fprintf(fp,"stroke\n0 0 0 setrgbcolor\n");
	
	fprintf(fp,"grestore\nshowpage\n%%%%EOF\n");
	fclose(fp);
      }
      for(QQ=4;QQ<=5;QQ++) {
	T1[0]='\0';
	strcat(T1,Aname[0][K]);
	if(Frame) {
	  T2[0]='\0';
	  fprintf(stderr,"%d",Frame);
	  sprintf(T2,"-f%1d",Frame);
	  strcat(T1,T2);
	}
	sprintf(T2,"-tri-%d.ps",QQ);	
	strcat(T1,T2);
	printf(" -> %s\n",T1);
	fp=fopen(T1,"w");
	AA = (char **) space(sizeof(char*)*5);
	print_ps_header(fp);
	ii=0;
      
	m1=0.0; m2=0.0; m3=0.0;
	v1=0.0; v2=0.0; v3=0.0;
	mm=0;
	for(j=0;j<n[4];j++) {
	  for(k=0;k<n[1];k++) {
	    for(l=0;l<n[2];l++) {
	      int i1;
	      AA[0] = A[0][K];
	      AA[1] = A[4][j];
	      AA[2] = A[1][k];
	      AA[3] = A[2][l];
	      s = tree_support(AA,noGaps,Frame);
	      P[ii][0] = s[0]; P[ii][1]=s[1]; ii++;	      
	      for(i1=0;i1<3;i1++) {
		fprintf(fp, "%8.6f ",s[i1]);
	      }
	      mm += 1.0;
	      m1 += s[0];      m2 += s[1];      m3 += s[2];
	      v1 += s[0]*s[0]; v2 += s[1]*s[1]; v3 += s[2]*s[2]; 
	      fprintf(fp, "sink\n");
	      free(s);
	    }
	  }
	}
	if(QQ==4) { 
	  sprintf(Aa,"%s",TIT[0]);
	  sprintf(Bb,"%s",TIT[4]);
	  sprintf(Cc,"%s",TIT[1]);
	  sprintf(Dd,"%s+%s",TIT[2],TIT[3]);
	  print_ps_header2(fp,Aa,Bb,Cc,Dd);	  
	  for(j=0;j<n[4];j++) {
	    for(k=0;k<n[1];k++) {
	      for(l=0;l<n[3];l++) {
		int i1;
		AA[0] = A[0][K];
		AA[1] = A[4][j];
		AA[2] = A[1][k];
		AA[3] = A[3][l];
		s = tree_support(AA,noGaps,Frame);
		P[ii][0] = s[0]; P[ii][1]=s[1]; ii++;		
		for(i1=0;i1<3;i1++) {
		  fprintf(fp, "%8.6f ",s[i1]);
		}
		mm += 1.0;
		m1 += s[0];      m2 += s[1];      m3 += s[2];
		v1 += s[0]*s[0]; v2 += s[1]*s[1]; v3 += s[2]*s[2]; 
		fprintf(fp, "sink\n");
		free(s);
	      }
	    }
	  }
	}
	if(QQ==5) {
	  sprintf(Aa,"%s",TIT[0]);
	  sprintf(Bb,"%s",TIT[4]);
	  sprintf(Cc,"%s+%s",TIT[1],TIT[3]);
	  sprintf(Dd,"%s",TIT[2]);
	  print_ps_header2(fp,Aa,Bb,Cc,Dd);	  
	  for(j=0;j<n[4];j++) {
	    for(k=0;k<n[3];k++) {
	      for(l=0;l<n[2];l++) {
		int i1;
		AA[0] = A[0][K];
		AA[1] = A[4][j];
		AA[2] = A[3][k];
		AA[3] = A[2][l];
		s = tree_support(AA,noGaps,Frame);
		P[ii][0] = s[0]; P[ii][1]=s[1]; ii++;
		for(i1=0;i1<3;i1++) {
		  fprintf(fp, "%8.6f ",s[i1]);
		}
		mm += 1.0;
		m1 += s[0];      m2 += s[1];      m3 += s[2];
		v1 += s[0]*s[0]; v2 += s[1]*s[1]; v3 += s[2]*s[2]; 
		fprintf(fp, "sink\n");
		free(s);
	      }
	    }
	  }
	}
	free(AA);
	print_triangle_data(fp, ii,P);
	m1=m1/mm; m2=m2/mm; m3=m3/mm;
	v1=v1/mm; v2=v2/mm; v3=v3/mm; 
	vv = (v1+v2+v3)/3.0;
	printf("QQ=%d\n",QQ);
	printf("T1: %6.4f +/- %6.4f  %s\n", m1, sqrt(v1-m1*m1),TIT[4] );
	printf("T2: %6.4f +/- %6.4f  %s", m2, sqrt(v2-m2*m2),TIT[1] );
	if(QQ==5) printf(" + %s\n",TIT[3]);else printf("\n");
	printf("T3: %6.4f +/- %6.4f  %s", m3, sqrt(v3-m3*m3),TIT[2] );
	if(QQ==4) printf(" + %s\n",TIT[3]);else printf("\n");
	
	fprintf(fp,"0 1 0 setrgbcolor\nnewpath\n");
	fprintf(fp,"%6.4f %6.4f %6.4f tricoord 0.015 0 360 arc\n",m1,m2,m3);
	fprintf(fp,"fill stroke\nnewpath\n");
	fprintf(fp,"%6.4f %6.4f %6.4f tricoord %6.4f 0 360 arc\n",m1,m2,m3,vv);
	fprintf(fp,"stroke\n0 0 0 setrgbcolor\n");
	
	fprintf(fp,"grestore\nshowpage\n%%%%EOF\n");
	fclose(fp);
      }
      QQ=6; {
	T1[0]='\0';
	strcat(T1,Aname[0][K]);
	if(Frame) {
	  T2[0]='\0';
	  fprintf(stderr,"%d\n",Frame);
	  sprintf(T2,"-f%1d",Frame);
	  strcat(T1,T2);
	}
	sprintf(T2,"-tri-%d.ps",QQ);	
	strcat(T1,T2);
	printf(" -> %s\n",T1);
	fp=fopen(T1,"w");
	AA = (char **) space(sizeof(char*)*5);
	print_ps_header(fp);
	ii=0;
	m1=0.0; m2=0.0; m3=0.0;
	v1=0.0; v2=0.0; v3=0.0;
	mm=0.0;
	for(j=0;j<n[4];j++) {
	  for(k=0;k<n[1];k++) {
	    for(l=0;l<n[3];l++) {
	      int i1;
	      AA[0] = A[0][K];
	      AA[1] = A[4][j];
	      AA[2] = A[1][k];
	      AA[3] = A[3][l];
	      s = tree_support(AA,noGaps,Frame);
	      P[ii][0] = s[0]; P[ii][1]=s[1]; ii++;
	      for(i1=0;i1<3;i1++) {
		fprintf(fp, "%8.6f ",s[i1]);
	      }
	      mm += 1.0;
	      m1 += s[0];      m2 += s[1];      m3 += s[2];
	      v1 += s[0]*s[0]; v2 += s[1]*s[1]; v3 += s[2]*s[2]; 
	      fprintf(fp, "sink\n");
	      free(s);
	    }
	  }
	}
	if(QQ==6) {
	  sprintf(Aa,"%s",TIT[0]);
	  sprintf(Bb,"%s",TIT[4]);
	  sprintf(Cc,"%s+%s",TIT[1],TIT[2]);
	  sprintf(Dd,"%s",TIT[3]);
	  print_ps_header2(fp,Aa,Bb,Cc,Dd);
	  for(j=0;j<n[4];j++) {
	    for(k=0;k<n[2];k++) {
	      for(l=0;l<n[3];l++) {
		int i1;
		AA[0] = A[0][K];
		AA[1] = A[4][j];
		AA[2] = A[2][k];
		AA[3] = A[3][l];
		s = tree_support(AA,noGaps,Frame);
		P[ii][0] = s[0]; P[ii][1]=s[1]; ii++;
		for(i1=0;i1<3;i1++) {
		  fprintf(fp, "%8.6f ",s[i1]);
		}
		mm += 1.0;
		m1 += s[0];      m2 += s[1];      m3 += s[2];
		v1 += s[0]*s[0]; v2 += s[1]*s[1]; v3 += s[2]*s[2]; 
		fprintf(fp, "sink\n");
		free(s);
	      }
	    }
	  }
	}
	free(AA);
	print_triangle_data(fp, ii,P);
	m1=m1/mm; m2=m2/mm; m3=m3/mm;
	v1=v1/mm; v2=v2/mm; v3=v3/mm; 
	vv = (v1+v2+v3)/3.0;
	printf("QQ=%d\n",QQ);
	printf("T1: %6.4f +/- %6.4f  %s\n", m1, sqrt(v1-m1*m1),TIT[4] );
	printf("T2: %6.4f +/- %6.4f  %s + %s\n", m2, sqrt(v2-m2*m2),
	       TIT[1],TIT[2] );
	printf("T3: %6.4f +/- %6.4f  %s\n", m3, sqrt(v3-m3*m3),TIT[3] );
	
	fprintf(fp,"0 1 0 setrgbcolor\nnewpath\n");
	fprintf(fp,"%6.4f %6.4f %6.4f tricoord 0.015 0 360 arc\n",m1,m2,m3);
	fprintf(fp,"fill stroke\nnewpath\n");
	fprintf(fp,"%6.4f %6.4f %6.4f tricoord %6.4f 0 360 arc\n",m1,m2,m3,vv);
	fprintf(fp,"stroke\n0 0 0 setrgbcolor\n");
	
	fprintf(fp,"grestore\nshowpage\n%%%%EOF\n");
	fclose(fp);
      }
    }
  }
  for(i=0;i<5;i++) {
    for(j=0;j<1999;j++) {
      if(A[i][j]!=NULL) free(A[i][j]);
      if(Aname[i][j]!=NULL) free(Aname[i][j]); 
    }
    free(A[i]);
    free(Aname[i]);
  }
  free(A);free(Aname);
  for(ii=0;ii<N;ii++) free(P[ii]);
  free(P);

  
  return 0;
}


static void print_ps_header(FILE *fp) {
  fprintf(fp,
	  "%%!PS\n"
	  "%%%%BoundingBox: 60 260 540 690\n"
	  "%%%%Title: 3Simplex-Trajectories\n"
	  "%%%%Author: (c) Peter F Stadler 1998\n\n"
	  "/sq34 { 0.8660254 } def\n"
	  "/sq3  { 1.7320508 } def\n"
	  "/the_triangle { triangle_line_width setlinewidth\n"
	  "            newpath 0 0 moveto 1 0 lineto 0.5 sq34 lineto\n"
	  "            closepath stroke\n"
	  "          } bind def\n"
	  "/tricoord { 3 2 roll pop dup sq34 mul \n"
	  "             3 1 roll 0.5 mul add exch \n"
	  "          } bind def\n"
	  "/sink     { triangle_line_width 0.6 mul setlinewidth\n"
	  "            newpath tricoord symbol_size 0 360 arc fill stroke\n"
	  "            triangle_line_width setlinewidth\n"
	  "          }   bind def\n"
	  "/source   { triangle_line_width 0.6 mul setlinewidth\n"
	  "            newpath tricoord symbol_size 0 360 arc\n"
	  "            gsave 1 setgray  fill grestore 0 setgray stroke\n"
	  "            triangle_line_width setlinewidth\n"
	  "          } bind def\n"
	  "/saddle   { triangle_line_width 0.6 mul setlinewidth\n"
	  "            newpath tricoord 2 copy symbol_size 0 360 arc\n"
	  "            gsave 1 setgray  fill grestore 0 setgray stroke\n"
	  "            2 copy\n"
	  "            newpath\n"
	  "            moveto\n"
	  "            0 symbol_size rmoveto\n"
	  "            0 symbol_size neg 2 mul rlineto stroke\n"
	  "            newpath\n"
	  "            moveto\n"
	  "            symbol_size 0 rmoveto\n"
	  "            symbol_size neg 2 mul 0 rlineto stroke\n"
	  "            triangle_line_width setlinewidth\n"
	  "          } bind def\n"
	  "/centerfp { triangle_line_width 0.6 mul setlinewidth\n"
	  "            newpath tricoord 2 copy symbol_size 0 360 arc\n"
	  "            gsave 1 setgray  fill grestore 0 setgray stroke\n"
	  "            newpath  symbol_size 0.4 mul 0 360 arc\n"
	  "            fill stroke\n"
	  "            triangle_line_width setlinewidth\n"
	  "          } bind def\n"
	  "/otherfp  { triangle_line_width 0.6 mul setlinewidth\n"
	  "            newpath tricoord symbol_size 0 360 arc\n"
	  "            gsave 0.9 setgray  fill grestore 0 setgray stroke\n"
	  "            triangle_line_width setlinewidth\n"
	  "          } bind def\n"
	  "/initialpoint { newpath tricoord 2 copy\n"
	  "            trajectory_line_width setlinewidth\n"
	  "            moveto\n"
	  "            0 symbol_size 0.5 mul rmoveto\n"
	  "            0 symbol_size 0.5 mul neg 2 mul rlineto stroke\n"
	  "            newpath\n"
	  "            2 copy\n"
	  "            moveto\n"
	  "            symbol_size 0.5 mul 0 rmoveto\n"
	  "            symbol_size 0.5 mul neg 2 mul 0 rlineto stroke\n"
	  "            newpath moveto\n"
	  "          } bind def\n"
	  "/end_trajectory { stroke triangle_line_width setlinewidth } bind def\n"
	  "/tl { tricoord lineto} bind def\n"
	  "/centershow { dup stringwidth pop 2 div neg 0 rmoveto show } bind def\n\n"
	  
	  "/the_grid {\n"
	  "triangle_line_width  3 div setlinewidth\n"
	  "0.166666 0.166666 0.666666 tricoord moveto\n"
	  "0.666666 0.166666 0.166666 tricoord lineto\n"
	  "0.75 0.00 0.25 tricoord lineto\n"
	  "0.666666 0.166666 0.166666 tricoord moveto\n"
	  "0.75 0.25 0.00 tricoord lineto\n"
	  "0.666666 0.166666 0.166666 tricoord moveto\n"
	  "0.166666 0.666666 0.166666 tricoord lineto\n"
	  "0.25 0.75 0.00 tricoord lineto\n"
	  "0.166666 0.666666 0.166666 tricoord moveto\n"
	  "0.00 0.75 0.25 tricoord lineto\n"
	  "0.166666 0.666666 0.166666 tricoord moveto\n"
	  "0.166666 0.166666 0.666666 tricoord lineto\n"
	  "0.25 0.00 0.75 tricoord lineto\n"
	  "0.166666 0.166666 0.666666 tricoord moveto\n"
	  "0.00 0.25 0.75 tricoord lineto stroke } bind def\n"
	  "gsave\n"
	  "100 300 translate\n"
	  "400 400 scale\n"
	  "%% User defined parameters\n"
	  "/triangle_line_width   { 0.005 } def   %% set line width of boundary\n"
	  "/symbol_size           { 0.005 } def   %% set size of fp symbols\n"
	  "/trajectory_line_width { 0.001 } def   %% set trajectory line width\n"
	  "\n"
	  "/Helvetica findfont 0.04 scalefont setfont\n"
	  "\n"
	  "the_triangle\n"
	  " 1 0 0 setrgbcolor\n"
	  );
}
static void print_ps_header2(FILE *fp, char*A, char *B, char *C, char *D) {
  fprintf(fp,"0 0 0 setrgbcolor\n");
  fprintf(fp,
	  "1.07000  0.00000 -0.07000 tricoord moveto (\\(\\(%s\\)\\(%s\\)|\\(%s\\)\\(%s\\)\\)) centershow\n",
	  A,B,C,D);
  fprintf(fp,  
	  "0.07000  1.00000 -0.07000 tricoord moveto (\\(\\(%s\\)\\(%s\\)|\\(%s\\)\\(%s\\)\\)) centershow\n",
	  A,C,B,D);
  fprintf(fp,
	  "0.07000  0.00000 1.03000 tricoord moveto (\\(\\(%s\\)\\(%s\\)|\\(%s\\)\\(%s\\)\\)) centershow\n",
	  A,D,B,C);
  fprintf(fp,"1 0 0 setrgbcolor\n");
}
  
				   
static void usage(char *who) {
  printf("%s [-g] [-F <1,2,3>] [-v] < input.qq > output \n",who);
  printf("%s -h   ... Help Message\n",who);
  printf("%s -V   ... Version Info\n",who);
  exit(0);
}




  

/* End of file */
