/* quart.c */
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <math.h>
#include "utils.h"

extern int *ch2d(double **PP, int N);

/* compute support for the tree different tree geometries */

double * tree_support( char **A , int NoGaps, int frame) {
  int i;
  int n;
  double *sigma;
  double s;

  sigma = (double *) space(3*sizeof(double));
  n = strlen(A[0]);
  for(i=1;i<4;i++){if(strlen(A[i])!=n){
    nrerror("Sequences of unequal length!");}}
  
  for(i=0;i<n;i++) {
    int ng,j;
    int useit;

    /* decide whether the column should be used */
    useit = 0;
    if(frame==0) useit = 1;
    if( (i%3)==(frame%3) ) { useit = 1; }
    ng=0;
    if(NoGaps) { for(j=0;j<4;j++) { if(A[j][i]=='-') { ng++; } } }
    if(ng>0) useit=0;
    /* */
    
    if (useit) { 
      
      /*  (01|23) */
      
      if(A[0][i]==A[1][i]) {
	if(A[2][i]==A[0][i]) continue;
	if(A[3][i]==A[0][i]) continue;
	sigma[0] +=1.0;
	if(A[2][i]==A[3][i]) sigma[0] += 1.0;
      }
      else if(A[2][i]==A[3][i]) {
	if(A[0][i]==A[2][i]) continue;
	if(A[1][i]==A[2][i]) continue;
	sigma[0] += 1.0;
      }
      
      /*  (02|13) */   	
      
      if(A[0][i]==A[2][i]) {
	if(A[1][i]==A[0][i]) continue;
	if(A[3][i]==A[0][i]) continue;
	sigma[1] +=1.0;
	if(A[1][i]==A[3][i]) sigma[1] += 1.0;
      }
      else if(A[1][i]==A[3][i]) {
	if(A[0][i]==A[1][i]) continue;
	if(A[2][i]==A[1][i]) continue;
	sigma[1] += 1.0;
      }
      
      /*  (03|21) */
      
      if(A[0][i]==A[3][i]) {
	if(A[1][i]==A[0][i]) continue;
	if(A[2][i]==A[0][i]) continue;
	sigma[2] +=1.0;
	if(A[2][i]==A[1][i]) sigma[2] += 1.0;
      }
      else if(A[2][i]==A[1][i]) {
	if(A[0][i]==A[2][i]) continue;
	if(A[3][i]==A[2][i]) continue;
	sigma[2] += 1.0;
      }
    }
  }
  s = sigma[0] +  sigma[1] + sigma[2];

  if(s>0.0) { for(i=0;i<3;i++) { sigma[i] /= s; } }
  else      { for(i=0;i<3;i++) { sigma[i] = 0.333333333333333; } }
  
  return sigma;
}

void print_triangle_data(FILE *fp, int N, double **PP){ 
  int i,j;
  int *idx;
  double m1, m2, m3, mm,  v1, v2, v3, vv;

  /* make statistics */
  m1=m2=m3=mm=v1=v2=v3=vv=0.0;  
  for(i=0;i<N;i++) {
    double PP3;
   
    PP3 = 1.0 - PP[i][0] - PP[i][1]; 
    mm += 1.0; 
    m1 += PP[i][0];      m2 += PP[i][1];      m3 += PP3;
    v1 += PP[i][0]*PP[i][0];
    v2 += PP[i][1]*PP[i][1];
    v3 += PP3*PP3; 
  }

  /* for(i=0;i<N;i++) printf("%8.6f %8.6f\n", PP[i][0],PP[i][1]); */

  /* convex hull */

  idx = ch2d(PP,N);
  fprintf(fp,"1.0 0.9 0.6  setrgbcolor\n");
  fprintf(fp,"newpath\n");
  i=0;
  j = idx[i];
  fprintf(fp,"%8.6f %8.6f %8.6f tricoord moveto\n",
	  PP[j][0],PP[j][1],1.0-PP[j][0]-PP[j][1]);
  i++;
  while(idx[i]>=0) {
    j = idx[i];
    fprintf(fp,"%8.6f %8.6f %8.6f tricoord lineto\n",
	    PP[j][0],PP[j][1],1.0-PP[j][0]-PP[j][1]);
    i++;
  }
  fprintf(fp,"closepath fill stroke\n");

  
  fprintf(fp,
	  "triangle_line_width  3 div setlinewidth\n"
	  "0.9 0.4 0.2 setrgbcolor\n"
	  "newpath\n");
  i=0;
  j = idx[i];
  fprintf(fp,"%8.6f %8.6f %8.6f tricoord moveto\n",
	  PP[j][0],PP[j][1],1.0-PP[j][0]-PP[j][1]);
  i++;
  while(idx[i]>=0) {
    j = idx[i];
    fprintf(fp,"%8.6f %8.6f %8.6f tricoord lineto\n",
	    PP[j][0],PP[j][1],1.0-PP[j][0]-PP[j][1]);
    i++;
  }
  fprintf(fp,"closepath stroke\n");  

  /* end convex hull related */

  /* Do the grid here! */

  fprintf(fp,
	  " 0 0 0 setrgbcolor\n"
	  "	  the_grid\n"
	  "triangle_line_width setlinewidth\n"
	  );
  
  /* printout */
  m1=m1/mm; m2=m2/mm; m3=m3/mm;
  v1=v1/mm-m1*m1; v2=v2/mm-m2*m2; v3=v3/mm-m3*m3; 
  vv = sqrt((v1+v2+v3)/2.0);
  fprintf(fp,"0 1 0 setrgbcolor\nnewpath\n");
  fprintf(fp,"%6.4f %6.4f %6.4f tricoord 0.015 0 360 arc\n",m1,m2,m3);
  fprintf(fp,"fill stroke\nnewpath\n");
  fprintf(fp,"%6.4f %6.4f %6.4f tricoord %6.4f 0 360 arc\n",m1,m2,m3,vv);
  fprintf(fp,"stroke\n0 0 0 setrgbcolor\n"); 
  fprintf(fp,"1 0 0 setrgbcolor\n"); 

  for(i=0;i<N;i++) {
    fprintf(fp, "%8.6f ", PP[i][0]);
    fprintf(fp, "%8.6f ", PP[i][1]);
    fprintf(fp, "%8.6f " , 1.0-PP[i][0]-PP[i][1] );
    fprintf(fp, "sink\n");
  }
    
  fprintf(fp,"grestore\nshowpage\n%%%%EOF\n");

  /* cleanup */

  free(idx);
 
  
}



/* End of file */
